#!/usr/bin/env python

import threading
import wx
import ut
import opt
import wfrm

def new_dclick():
	e = ut.Empty()
	e.timer = None
	e.pos = None
	
	def cancel():
		if e.timer:
			e.timer.cancel()
			e.timer = None
	def check(event):
		typeId = event.GetEventType()
		if typeId != wx.EVT_LEFT_UP.typeId:
			cancel()
			return False

		pos = event.GetPosition()
		if not e.timer:
			e.pos = pos
			e.timer = threading.Timer(0.3, cancel)
			e.timer.start()
			return False
		cancel()
		return pos == e.pos
	e.check = check
	return e

class MyFrame(wx.Frame):
	def __init__(self, *args, **kwds):
		kwds["style"] = wx.DEFAULT_FRAME_STYLE
		wx.Frame.__init__(self, *args, **kwds)

		self.SetSize((320,240))
		self.panel = wx.Panel(self, wx.ID_ANY)
		sizer_1 = wx.BoxSizer(wx.VERTICAL)
		sizer_1.Add(self.panel, 1, wx.EXPAND, 0)
		self.SetSizer(sizer_1)
		self.Layout()

		evs = (
			wx.EVT_MOTION,
			wx.EVT_LEFT_DOWN,
			wx.EVT_LEFT_UP,
			wx.EVT_MIDDLE_DOWN,
			wx.EVT_MIDDLE_UP,
			wx.EVT_RIGHT_DOWN,
			wx.EVT_RIGHT_UP,
		)
		for ev in evs:
			self.panel.Bind(ev, self.hdr)
		self.panel.Bind(wx.EVT_PAINT, self.OnPaint)

		fn_yaml = opt.get('-fn', 'cube.yaml')
		self.wf = wfrm.new( fn_yaml, self.panel.GetSize )
		self.dclick = new_dclick()

	def OnPaint(self, event):
		lst = self.wf.get_lst()
		dc = wx.PaintDC(self.panel)
		for (x1, y1, x2, y2) in lst:
			dc.DrawLine(x1, y1, x2, y2)

	def hdr(self, event):
		if self.dclick.check(event):
			self.wf.reset()
			self.panel.Refresh()
			return

		typeId = event.GetEventType()
		(x, y) = event.GetPosition()
		d = {
			wx.EVT_RIGHT_DOWN.typeId: 'zoom',
			wx.EVT_MIDDLE_DOWN.typeId: 'move',
			wx.EVT_LEFT_DOWN.typeId: 'rot',
		}
		u = [
			wx.EVT_RIGHT_UP.typeId,
			wx.EVT_MIDDLE_UP.typeId,
			wx.EVT_LEFT_UP.typeId,
		]
		if typeId in d:
			self.wf.down(x, y, d.get(typeId) )
		elif typeId in u:
			self.wf.up()
		elif typeId == wx.EVT_MOTION.typeId and self.wf.mode:
			self.wf.drag(x, y)
			self.panel.Refresh()

class MyApp(wx.App):
	def OnInit(self):
		frame_1 = MyFrame(None, wx.ID_ANY, "")
		self.SetTopWindow(frame_1)
		frame_1.Show()
		return 1

if __name__ == "__main__":
	app = MyApp(0)
	app.MainLoop()
# EOF
