#!/usr/bin/env python

import math
import numpy as np
import ut
import mt
import v

def new(v3):
	# v3: [ v.x1, v.y1, v.z1 ]

	e = ut.Empty()
	e.typ = 'vecs'
	e.v3 = v3
	e.l2g = l2g = lambda v_: np.dot( v3.T, v_ )
	#e.g2l = g2l = lambda v: e.rev().tr('l2g', v)
	e.g2l = g2l = lambda v: e.rev().l2g(v)
	e.tr = lambda d, v: { 'l2g': l2g, 'g2l': g2l }.get(d, l2g)(v)

	e.rev_vs = None
	def rev():
		if not e.rev_vs:
			e.rev_vs = new( np.linalg.inv(v3) )
		return e.rev_vs
	e.rev = rev

	e.zoom = lambda zm3: new( v3 * mt.T( mt.arr(zm3) ) )
	e.zoom_all = lambda zm: e.zoom( [zm,zm,zm] )
	e.zoom_x = lambda zm: e.zoom( [zm,1,1] )
	e.zoom_y = lambda zm: e.zoom( [1,zm,1] )
	e.zoom_z = lambda zm: e.zoom( [1,1,zm] )

	e.str = lambda : str(e.v3)

	return e

one = new( mt.arr( [v.x1, v.y1, v.z1] ) )

def by_fix_ref(fix_nm, fix, ref_nm, ref):
	fix_i = 'xyz'.index(fix_nm)
	ref_i = 'xyz'.index(ref_nm)

	lst = [0,1,2]
	lst.remove(fix_i)
	lst.remove(ref_i)
	i = lst[0]

	v3 = [ [],[],[] ]
	v3[fix_i] = fix

	if all( fix == ref ):
		ref = ( lambda x, y, z: (-y, x, z) )( *ref ) if all( ref != v.z1 ) else v.x1

	(a, b) = (fix, ref) if (fix_i + 1) % 3 == ref_i else (ref, fix)
	v3[i] = c = v.cross_product_unit(a, b)

	(a, b) = (fix, c) if (fix_i + 1) % 3 == i else (c, fix)
	v3[ref_i] = v.cross_product_unit(a, b)
	
	return new( mt.arr(v3) )

def rot_y(deg):
	rad = mt.deg_to_rad(deg)
	s = math.sin(rad)
	c = math.cos(rad)

	(x, y, z) = one.v3
	(xx, xy, xz) = x
	(yx, yy, yz) = y
	(zx, zy, zz) = z

	zz = xx = c
	xz = -s
	zx = s

	x = (xx, xy, xz)
	y = (yx, yy, yz)
	z = (zx, zy, zz)
	v3 = (x, y, z)

	return new( mt.arr(v3) )
# EOF
