#!/usr/bin/env python

import yaml
import math
import ut
import mt
import v
import line
import ax
import fcx
import lstx

def nv_clip(nv, pa, pb):
	nv_line = line.new( mt.zero, nv )
	(in_a, in_b) = ut.map_lst( lambda p: nv_line.is_plane_side(p), [pa, pb] )
	if in_a and in_b:
		return [pa, pb]
	if not in_a and not in_b:
		return []
	pc = nv_line.cross_plane_line( line.new_p2(pa, pb) )
	return [pa, pc] if in_a else [pc, pb]

def eye_clip(eye2g, wh2eye, sc_sz, pa, pb):
	(w, h, d) = sc_sz
	n = 4
	p4 = mt.arr( [ [0,0,-d], [w,0,-d], [w,h,-d], [0,h,-d] ] )
	p4 = ut.map_lst( lambda p: lstx.tr(wh2eye, 'l2g', p), p4 )
	for i in range(n):
		nv = v.p3_to_nv( mt.zero, p4[i], p4[(i+1)%n] )
		r = nv_clip(nv, pa, pb)
		if not r:
			return []
		(pa, pb) = r
	return [pa, pb]

def cvt(ps, idxs, eye2g, wh2eye, sc_sz):
	ps = ut.map_lst( lambda p: eye2g.tr('g2l', p), ps )
	p2s = ut.map_up( lambda i, j: eye_clip( eye2g, wh2eye, sc_sz, ps[i], ps[j] ), idxs )
	p2s = filter( lambda p2: p2, p2s )
	p2s = map( lambda p2: ut.map_lst( lambda p: lstx.tr( wh2eye, 'g2l', p), p2 ), p2s )

	def f(p2):
		(ax, ay, _) = p2[0]
		(bx, by, _) = p2[1]
		return ( int(ax), int(ay), int(bx), int(by) )

	return list( map( f, p2s ) )

def load(fn_yaml):
	f = open(fn_yaml, 'r')
	s = f.read()
	f.close()
	d = yaml.load(s)

	ks = ( 'ps', 'idxs', 'eye_p', 'eye_t' )
	( ps, idxs, eye_p, eye_t ) = map( d.get, ks )
	( ps, eye_p, eye_t ) = map( mt.arr, ( ps, eye_p, eye_t ) )

	f = lambda lst: list( map( lambda i: [ lst[i], lst[i+1] ], range( len(lst) - 1 ) ) )
	idxs = sum( map( f, idxs ), [] )

	return (ps, idxs, eye_p, eye_t)

def new(fn_yaml, f_wh, w_deg=30):
	( ps, idxs, eye_p, eye_t ) = load(fn_yaml)

	e = ut.Empty()
	e.p = eye_p
	e.t = eye_t
	e.org = ( e.p, e.t )
	e.mode = ''

	dz = 1.0 / math.tan( 0.5 * mt.deg_to_rad(w_deg) )

	def get_lst():
		(w, h) = f_wh()
		sc_sz = (w, h, dz*w)
		wh2eye = lstx.opt( [ ax.wh_map(w, h), fcx.new(dz), ax.zoom_z(-1), ax.rot_x(90) ] )
		eye2g = ax.new_line( line.new_p2( e.p, e.t ) )
		return cvt(ps, idxs, eye2g, wh2eye, sc_sz)
	e.get_lst = get_lst

	def down(x, y, mode):
		e.bak = ( e.p.copy(), e.t.copy(), x, y )
		e.mode = mode
	e.down = down

	def up():
		e.mode = ''
	e.up = up

	def drag(x, y):
		(bp, bt, bx, by) = e.bak
		(w, h) = f_wh()
		(w2, h2) = ( w*0.5, h*0.5 )
		(rx, ry) = ( (x - bx) / w2, (y - by) / h2 )
		if e.mode == 'zoom':
			e.p = bp + (-ry) * (bt - bp)
		elif e.mode == 'move':
			dzw = dz * w
			rx = math.atan2(x - w2, dzw) - math.atan2(bx - w2, dzw)
			ry = math.atan2(y - h2, dzw) - math.atan2(by - h2, dzw)
			ax_p = ax.new_line( line.new_p2( bp, bt ) )
			e.t = lstx.tr( [ ax_p, ax.rot_z(rx*90), ax.rot_x(ry*90), ax_p.rev() ], 'g2l', bt )
		elif e.mode == 'rot':
			ax_t = ax.new_line( line.new_p2( bt, bp ) )
			e.p = lstx.tr( [ ax_t, ax.rot_z(rx*90), ax.rot_x(-ry*90), ax_t.rev() ], 'g2l', bp )
	e.drag = drag

	def reset():
		(e.p, e.t) = e.org
	e.reset = reset

	return e
# EOF
